<?php
/**
 * Fichier gérant l'installation et la désinstallation du plugin.
 *
 * @package    SPIP\ISOCODE\ADMINISTRATION
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Fonction d'installation et de mise à jour du plugin.
 * Le schéma du plugin est composé des tables `spip_iso639xxxx` et d'une configuration.
 *
 * @param string $nom_meta_base_version Nom de la meta informant de la version du schéma de données du plugin installé dans SPIP
 * @param string $version_cible         Version du schéma de données (déclaré dans paquet.xml)
 *
 * @return void
 **/
function isocode_upgrade(string $nom_meta_base_version, string $version_cible) : void {
	$maj = [];

	// Liste des tables créées par le plugin
	include_spip('inc/isocode');
	$tables = [];
	foreach (isocode_lister_types_service() as $_type) {
		foreach (isocode_lister_tables($_type) as $_table) {
			$tables[] = "spip_{$_table}";
		}
	}

	$maj['create'] = [
		[
			'maj_tables',
			$tables
		],
	];

	$maj['2'] = [
		[
			'maj_tables',
			['spip_infrasubdivisions']
		],
		['sql_alter', "TABLE spip_iso3166alternates CHANGE code_iso code_iso varchar(20) DEFAULT '' NOT NULL"],
		['sql_alter', "TABLE spip_iso3166alternates CHANGE code_alter code_alter varchar(20) DEFAULT '' NOT NULL"],
	];

	$maj['3'] = [
		['sql_alter', 'TABLE spip_m49regions ADD COLUMN depth int DEFAULT 0 NOT NULL AFTER parent'],
		['sql_alter', 'TABLE spip_iso3166subdivisions ADD COLUMN depth int DEFAULT 0 NOT NULL AFTER parent'],
		['sql_alter', 'TABLE spip_infrasubdivisions ADD COLUMN depth int DEFAULT 0 NOT NULL AFTER parent'],
	];

	$maj['4'] = [
		['sql_alter', 'TABLE spip_infrasubdivisions ADD COLUMN postcode varchar(10) DEFAULT "" NOT NULL AFTER label'],
		['sql_alter', 'TABLE spip_infrasubdivisions ADD COLUMN lat double NULL NULL AFTER postcode'],
		['sql_alter', 'TABLE spip_infrasubdivisions ADD COLUMN lon double NULL NULL AFTER lat'],
	];

	$maj['5'] = [
		['sql_alter', 'TABLE spip_infrasubdivisions ADD COLUMN parent_alt1 varchar(20) DEFAULT "" NOT NULL AFTER depth'],
		['sql_alter', 'TABLE spip_infrasubdivisions ADD COLUMN parent_alt2 varchar(20) DEFAULT "" NOT NULL AFTER parent_alt1'],
	];

	$maj['6'] = [
		['sql_alter', 'TABLE spip_m49regions MODIFY category varchar(64) DEFAULT "" NOT NULL'],
		['sql_alter', 'TABLE spip_iso3166countries MODIFY category varchar(64) DEFAULT "" NOT NULL'],
		['sql_alter', 'TABLE spip_iso3166subdivisions MODIFY type varchar(64) DEFAULT "" NOT NULL'],
		['sql_alter', 'TABLE spip_infrasubdivisions MODIFY type varchar(64) DEFAULT "" NOT NULL'],
	];

	include_spip('base/upgrade');
	maj_plugin($nom_meta_base_version, $version_cible, $maj);
}


/**
 * Fonction de désinstallation du plugin.
 *
 * @param string $nom_meta_base_version Nom de la meta informant de la version du schéma de données du plugin installé dans SPIP.
 *
 * @return void
 **/
function isocode_vider_tables(string $nom_meta_base_version) : void {
	// Supprimer les tables ISO créées par le plugin
	include_spip('inc/isocode');
	foreach (isocode_lister_types_service() as $_type) {
		foreach (isocode_lister_tables($_type) as $_table) {
			sql_drop_table("spip_{$_table}");
		}
	}

	// Effacer la meta de configuration et de stockage du plugin
	effacer_meta('isocode');

	// Effacer la meta du schéma de la base
	effacer_meta($nom_meta_base_version);
}
