<?php
/**
 * Ce fichier contient l'ensemble des fonctions implémentant l'API du plugin.
 *
 * @package SPIP\ISOCODE\API
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Informe sur la liste des tables déjà chargées en base de données pour un type de service donné.
 * Les informations de la meta de chaque consignation sont complétées et renvoyées.
 * Si le type est `nomenclature`, les informations sont renvoyées en deux groupes `geometrie` et `langue``
 *
 * @api
 *
 * @filtre
 *
 * @param string      $type   Type de service. Prend les valeurs `nomenclature`, `infra_subdivision` ou `geometrie`.
 *
 * @return array Liste des consignations de chargement.
 */
function isocode_informer_consignation(string $type) : array {
	// Initialisation des chargements
	$chargements = [];

	// On initialise la liste des tables en lisant la meta idoine.
	include_spip('inc/config');
	$consignations = lire_config("isocode/{$type}", []);

	// On complète chaque bloc d'informations
	if ($consignations) {
		foreach (array_keys($consignations) as $_cle) {
			// Libellé du type
			$libelle_type = _T("isocode:{$type}_{$_cle}");
			if ($type === 'nomenclature') {
				$groupe = $consignations[$_cle]['groupe'];
				$chargements[$groupe][$_cle] = $consignations[$_cle];
				$chargements[$groupe][$_cle]['libelle'] = $libelle_type;
			} else {
				$chargements[$_cle] = $consignations[$_cle];
				$chargements[$_cle]['libelle'] = $libelle_type;
			}
		}
	}

	return $chargements;
}
