<?php
/**
 * Ce fichier contient l'ensemble des constantes et fonctions implémentant le type de service GEOMETRIE.
 * Les services de GEOMETRIE sont des mécanisme internes à Nomenclatures pour
 * charger les différents ensemble de contours géographiques fournis par des services divers et sous des
 * formats variés (fichiers geoJSON, topoJSON, API REST...).
 *
 * @package SPIP\ISOCODE\SERVICES\GEOMETRIE
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

require_once _DIR_PLUGIN_ISOCODE . 'vendor/autoload.php';
use Vicchi\GeoJson\Rewind;

$GLOBALS['isocode']['geometrie'] = [
	'continents' => [
		'table'        => 'geoboundaries',
		'type'         => 'zone',
		'pays'         => '',
		'basic_fields' => [
			'm49'      => 'code',
			'geometry' => 'geometry',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
		],
		'populating'  => 'file_geojson',
		'multiple'    => false,
		'extension'   => '.json',
		'node'        => 'features',
		'basic_nodes' => [
			'm49'      => 'properties/m49',
			'geometry' => 'geometry',
		],
		'credits' => [
			'provider_name' => 'hrbrmstr, boB Rudis',
			'provider_url'  => 'https://gist.github.com/hrbrmstr/91ea5cc9474286c72838/',
			'license'       => 'MIT',
		],
	],
	'countries' => [
		'table'        => 'geoboundaries',
		'type'         => 'country',
		'pays'         => '',
		'basic_fields' => [
			'ISO_A2'      => 'code',
			'ISO_A2_EH'   => 'code_eh',
			'geometry'    => 'geometry',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'code_eh' => '',
		],
		'populating'  => 'file_geojson',
		'multiple'     => true,
		'extension'   => '.geojson',
		'node'        => 'features',
		'basic_nodes' => [
			'ISO_A2'    => 'properties/ISO_A2',
			'ISO_A2_EH' => 'properties/ISO_A2_EH',
			'geometry'  => 'geometry',
		],
		'credits' => [
			'provider_name' => 'Natural Earth',
			'provider_url'  => 'https://www.naturalearthdata.com/downloads/50m-cultural-vectors/',
			'license'       => 'Domaine Public',
			'last_update'   => '13/05/2022'
		],
	],
	'frregion' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'FR',
		'basic_fields' => [
			'reg_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_insee_reg',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-france-region&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'reg_code'     => 'fields/reg_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-france-region',
			'provider_name' => 'IGN',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-france-region/information/?disjunctive.reg_name',
			'license'       => 'Licence Ouverte 2.0 (Etalab)',
			'last_update'   => '01/12/2020'
		],
	],
	'frdepartment' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'FR',
		'basic_fields' => [
			'dep_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_insee',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-france-departement&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'dep_code'     => 'fields/dep_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-france-departement',
			'provider_name' => 'IGN',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-france-departement/information/?disjunctive.reg_name',
			'license'       => 'Licence Ouverte 2.0 (Etalab)',
			'last_update'   => '22/06/2020'
		],
	],
	'deland' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'DE',
		'basic_fields' => [
			'lan_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_ars',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-germany-land&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'lan_code'     => 'fields/lan_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-france-departement',
			'provider_name' => 'GeoBasis-DE / BKG 2020',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-germany-land/information/?disjunctive.reg_name',
			'license'       => 'Data licence Germany – attribution – version 2.0',
			'last_update'   => '11/09/2020'
		],
	],
	'chcanton' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'CH',
		'basic_fields' => [
			'kan_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_ktnr',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-switzerland-kanton&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'kan_code'     => 'fields/kan_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-switzerland-kanton',
			'provider_name' => 'Office fédéral de topographie swisstopo',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-switzerland-kanton/information/?disjunctive.kan_code&disjunctive.kan_name&sort=year',
			'license'       => 'Opendata.swiss BY ASK',
			'last_update'   => '05/01/2021'
		],
	],
	'beregion' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'BE',
		'basic_fields' => [
			'reg_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_ins_reg',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-belgium-region&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'reg_code'     => 'fields/reg_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-belgium-region',
			'provider_name' => 'National Geographic Institute (NGI-IGN, ngi.be)',
			'provider_url'  => '',
			'license'       => 'IGN Belge - ngi-standard-open',
			'last_update'   => '01/09/2022'
		],
	],
	'beprovince' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'BE',
		'basic_fields' => [
			'prov_code'    => 'code',      // Code INS fourni dans l'api
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_ins',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-belgium-province&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'prov_code'    => 'fields/prov_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-belgium-province',
			'provider_name' => 'National Geographic Institute (NGI-IGN, ngi.be)',
			'provider_url'  => '',
			'license'       => 'IGN Belge - ngi-standard-open',
			'last_update'   => '01/09/2022'
		],
	],
	'usstate' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'US',
		'basic_fields' => [
			'stusab'       => 'code',
			'st_asgeojson' => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=us-state-boundaries&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'stusab'       => 'fields/stusab',
			'st_asgeojson' => 'fields/st_asgeojson',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'us-state-boundaries',
			'provider_name' => 'U.S. Census Bureau',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/us-state-boundaries/information/?flg=fr',
			'license'       => 'Domaine public',
			'last_update'   => '24/10/2019'
		],
	],
	'nlprovince' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'NL',
		'basic_fields' => [
			'prov_code'    => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_cbs',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-netherlands-provincie&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'prov_code'    => 'fields/prov_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-netherlands-provincie',
			'provider_name' => 'Basisregistratie Kadaster',
			'provider_url'  => 'https://www.kadaster.nl/zakelijk',
			'license'       => 'CC 0 1.0',
			'last_update'   => '20/10/2021'
		],
	],
	'ptdistrict' => [
		'table'        => 'geoboundaries',
		'type'         => 'subdivision',
		'pays'         => 'PT',
		'basic_fields' => [
			'dis_code'     => 'code',
			'geo_shape'    => 'geometry',
			'geo_point_2d' => 'lat_lon',
		],
		'basic_ext_fields' => [
			'lat',
			'lon',
		],
		'static_fields' => [
			'service'   => '$service',
			'code_type' => 'code_iso',
			'format'    => 'geojson',
			'type'      => '/type',
			'country'   => '/pays',
		],
		'unused_fields' => [
			'lat_lon' => '',
		],
		'populating'  => 'api_rest',
		'url'         => 'https://public.opendatasoft.com/api/records/1.0/search/?dataset=georef-portugal-distrito&q=&rows=-1',
		'extension'   => '.json',
		'node'        => 'records',
		'basic_nodes' => [
			'dis_code'     => 'fields/dis_code',
			'geo_shape'    => 'fields/geo_shape',
			'geo_point_2d' => 'fields/geo_point_2d',
		],
		'credits' => [
			'api'           => 'OpenDataSoft',
			'dataset'       => 'georef-portugal-distrito',
			'provider_name' => 'Direção-Geral do Território',
			'provider_url'  => 'https://public.opendatasoft.com/explore/dataset/georef-portugal-distrito/information/?disjunctive.dis_code&disjunctive.dis_name&sort=year',
			'license'       => 'Domaine public',
			'last_update'   => '07/01/2021'
		],
	],
];

// ----------------------------------------------------------------------------
// ---------- API du type de service GEOMETRIE - Actions principales ----------
// ----------------------------------------------------------------------------

/**
 * Complète une élément extrait de la source via le service `continents` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function continents_completer_element(array $element, array $config) : array {
	// Apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geometry'] = json_encode(Rewind::rewind($element['geometry'], true));

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `countries` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function countries_completer_element(array $element, array $config) : array {
	// Cette fonction permet d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geometry'] = json_encode(Rewind::rewind($element['geometry'], true));

	// Corriger une erreur dans le fichier où le code ISO vaut -99 : dans ce cas on va chercher le code ISO_A2_EH
	if ($element['ISO_A2'] === "-99") {
		$element['ISO_A2'] = $element['ISO_A2_EH'];
	}

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `continents` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function frregion_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `frdepartment` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function frdepartment_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `deland` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function deland_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `chcanton` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function chcanton_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `beregion` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function beregion_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	// Formater le code INS sur 5 digits
	$element['reg_code'] = str_pad($element['reg_code'], 5, '0', STR_PAD_LEFT);

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `beprovince` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function beprovince_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	// Formater le code INS sur 5 digits
	$element['prov_code'] = str_pad($element['prov_code'], 5, '0', STR_PAD_LEFT);

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `usstate` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function usstate_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['st_asgeojson'] = json_encode(Rewind::rewind($element['st_asgeojson'], true));

	// Formater le code ISO en lui rajoutant le code ISO-3166 alpha2 des US
	$element['stusab'] = $element['country'] . '-' . $element['stusab'];

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `nlprovince` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function nlprovince_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `ptdistrict` (avant son évolution en enregistrement
 * de la table `geoboundaries`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function ptdistrict_completer_element(array $element, array $config) : array {
	// Cette fonction permet de remplir les champs "basic_ext".
	// - récupérer la latitude et la longitude à partir du champ geo_point_2d
	$element['lat'] = (float) ($element['geo_point_2d'][0]);
	$element['lon'] = (float) ($element['geo_point_2d'][1]);

	// - imposer le format Right-Hand Rule au contour
	// - encoder le champs des géométries
	$element['geo_shape'] = json_encode(Rewind::rewind($element['geo_shape'], true));

	// Formater le code ISO en lui rajoutant le code ISO-3166 alpha2 de la Belgique
	$element['dis_code'] = $element['country'] . '-' . $element['dis_code'];

	return $element;
}
