<?php
/**
 * Ce fichier contient l'ensemble des constantes et fonctions implémentant le type de service INFRASUBDIVISION.
 * Les services de INFRASUBDIVISION sont des mécanisme internes à Nomenclatures pour
 * charger les codes nationaux des infra subdivisions fournis par des services divers.
 *
 * @package SPIP\ISOCODE\SERVICES\INFRASUBDIVISION
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_ISOCODE_INSEECOMMUNE_LATLON')) {
	/**
	 * Chemin du fichier contenant les coordonnées géographiques des communes fraançaises.
	 * Complément à la table infrasubdivisions.
	 */
	define('_ISOCODE_INSEECOMMUNE_LATLON', 'services/infrasubdivision/inseecommune-latlon.csv');
}

$GLOBALS['isocode']['infrasubdivision'] = [
	'inseearrond' => [
		'table'        => 'infrasubdivisions',
		'type'         => 'infra_subdivision',
		'pays'         => 'FR',
		'basic_fields' => [
			'ARR'    => 'code',
			'DEP'    => 'parent',
			'NCCENR' => 'label_fr',
		],
		'static_fields' => [
			'service' => '$service',
			'type'    => 'arrondissement',
			'country' => '/pays',
		],
		'depth_field' => [
			'name'         => 'depth',
			'base_value'   => 0,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		],
		'unused_fields' => [
			'label_fr' => '',
		],
		'label_field' => true,
		'populating'  => 'file_csv',
		'multiple'    => true,
		'delimiter'   => ',',
		'extension'   => '.csv',
	],
	'inseeepci' => [
		'table'        => 'infrasubdivisions',
		'type'         => 'infra_subdivision',
		'pays'         => 'FR',
		'basic_fields' => [
			'EPCI'        => 'code',
			'NATURE_EPCI' => 'type',
			'DEP'         => 'parent',
			'LIBEPCI'     => 'label_fr',
		],
		'static_fields' => [
			'service' => '$service',
			'country' => '/pays',
		],
		'depth_field' => [
			'name'         => 'depth',
			'base_value'   => 0,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		],
		'unused_fields' => [
			'label_fr' => '',
		],
		'label_field' => true,
		'populating'  => 'file_csv',
		'multiple'    => true,
		'delimiter'   => ';',
		'extension'   => '.csv',
	],
	'inseecommune' => [
		'table'        => 'infrasubdivisions',
		'type'         => 'infra_subdivision',
		'pays'         => 'FR',
		'basic_fields' => [
			'TYPECOM'   => 'type',
			'COM'       => 'code',
			'DEP'       => 'dep',
			'ARR'       => 'parent',
			'NCCENR'    => 'label_fr',
			'CAN'       => 'can',
			'COMPARENT' => 'comparent',
			'EPCI'      => 'parent_alt1'
		],
		'static_fields' => [
			'service' => '$service',
			'country' => '/pays',
		],
		'depth_field' => [
			'name'         => 'depth',
			'base_value'   => 1,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		],
		'unused_fields' => [
			'dep'       => '',
			'can'       => '',
			'comparent' => '',
			'label_fr'  => '',
		],
		'addon_fields' => [
			'latlon' => [
				'lat' => 'lat',
				'lon' => 'lon',
			],
			'poste' => [
				'code' => 'postcode',
			],
		],
		'label_field' => true,
		'populating'  => 'file_csv',
		'multiple'    => true,
		'delimiter'   => ';',
		'extension'   => '.csv',
	],
	'uscbcounty' => [
		'table'        => 'infrasubdivisions',
		'type'         => 'infra_subdivision',
		'pays'         => 'US',
		'basic_fields' => [
			'code'   => 'code',
			'state'  => 'parent',
			'name'   => 'label_en',
			'entity' => 'type',
		],
		'static_fields' => [
			'service' => '$service',
			'country' => '/pays',
		],
		'depth_field' => [
			'name'         => 'depth',
			'base_value'   => 0,
			'parent_field' => 'parent',
			'parent_value' => 'code',
		],
		'unused_fields' => [
			'label_en' => '',
		],
		'label_field' => true,
		'populating'  => 'file_csv',
		'multiple'    => true,
		'delimiter'   => ';',
		'extension'   => '.csv',
	],
];

// -----------------------------------------------------------------------------------
// ---------- API du type de service INFRASUBDIVISION - Actions principales ----------
// -----------------------------------------------------------------------------------

/**
 * Complète une élément extrait de la source via le service `inseearrond` (avant son évolution en enregistrement
 * de la table `infrasubdivisions`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function inseearrond_completer_element(array $element, array $config) : array {
	// Cette fonction permet d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - modifier l'identifiant de l'arrondissement
	$element['ARR'] = 'FR-' . $element['ARR'];
	// - Modifier l'identifiant du parent qui est une subdivision
	//   Pour Paris, 75 on utilise le nouveau code ISO, '75C'
	if ((int) $element['DEP'] === 75) {
		$element['DEP'] = '75C';
	}
	$element['DEP'] = 'FR-' . $element['DEP'];

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `inseeepci` (avant son évolution en enregistrement
 * de la table `infrasubdivisions`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function inseeepci_completer_element($element, $config) {
	// Cette fonction permet d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - modifier l'identifiant de l'EPCI
	$element['EPCI'] = 'FR-' . $element['EPCI'];
	// - Modifier l'identifiant du parent qui est une subdivision
	if ((int) $element['DEP'] === 75) {
		$element['DEP'] = '75C';
	}
	$element['DEP'] = 'FR-' . $element['DEP'];
	// - affiner l'identifiant du type d'EPCI
	switch ($element['NATURE_EPCI']) {
		case 'CC':
			$element['NATURE_EPCI'] = 'municipality_community';
			break;
		case 'CA':
			$element['NATURE_EPCI'] = 'agglomeration_community';
			break;
		case 'CU':
			$element['NATURE_EPCI'] = 'urban_community';
			break;
		case 'ME':
			$element['NATURE_EPCI'] = 'metropolis';
			break;
		default:
			$element['NATURE_EPCI'] = 'epci';
	}

	return $element;
}

/**
 * Complète une élément extrait de la source via le service `inseecommune` (avant son évolution en enregistrement
 * de la table `infrasubdivisions`).
 *
 * @param array $element Elément extrait de la source.
 * @param array $config  Configuration du service.
 *
 * @return array Elément complété.
 */
function inseecommune_completer_element($element, $config) {
	// Cette fonction permet d'apporter des corrections au champs déjà compilés : attention on a encore les index source !!!
	// - modifier l'identifiant de l'arrondissement
	$element['COM'] = 'FR-' . $element['COM'];
	// - choisir le "bon" parent et modifier son identifiant
	if ($element['COMPARENT']) {
		$element['ARR'] = 'FR-' . $element['COMPARENT'];
	} elseif ($element['ARR']) {
		$element['ARR'] = 'FR-' . $element['ARR'];
	} else {
		$element['ARR'] = 'FR-' . $element['DEP'];
	}
	// - modifier l'identifiant de l'EPCI si besoin
	if ($element['EPCI']) {
		$element['EPCI'] = 'FR-' . $element['EPCI'];
	}
	// - affiner l'identifiant du type de commune
	switch ($element['TYPECOM']) {
		case 'COMD':
			$element['TYPECOM'] = 'delegated_commune';
			break;
		case 'COMA':
			$element['TYPECOM'] = 'associated_commune';
			break;
		case 'ARM':
			$element['TYPECOM'] = 'municipal_arrondissement';
			break;
		default:
			$element['TYPECOM'] = 'commune';
	}

	return $element;
}

/**
 * Complète les enregistrements de la table `infrasubdivisions` pour le service inseecommune.
 *
 * @param array $enregistrements Liste des enregistrements déjà remplis à partir de la source.
 * @param array $config          Configuration du service
 *
 * @return array Liste des enregistrements complétés.
 */
function inseecommune_completer_table($enregistrements, $config) {
	// Inclusion de l'API.
	include_spip('inc/isocode_utils');

	// Lecture du fichier CSV de la latitude et longitude.
	// Le délimiteur est un point-virgule.
	// -- Initialisation des champs additionnels
	$enregistrements_geo = [];
	$config_champs_geo = $config['addon_fields']['latlon'];
	$enregistrement_geo_defaut = initialiser_enregistrement('infrasubdivisions', $config_champs_geo);

	// -- Lecture du fichier des coordonnées géographiques
	$fichier = find_in_path(_ISOCODE_INSEECOMMUNE_LATLON);
	$separateur = ';';
	$lignes = file($fichier);
	if ($lignes) {
		$titres = [];
		$index_code_insee = null;
		foreach ($lignes as $_numero => $_ligne) {
			$valeurs = explode($separateur, trim($_ligne, "\r\n"));
			if ($_numero == 0) {
				// Stockage des noms de colonnes car la première ligne contient toujours le header et de
				// l'index correspondant au code INSEE de la commune qui se nomme "iso" dans le fichier CSV.
				$titres = $valeurs;
				$index_code_insee = array_search('iso', $titres);
			} else {
				// On extrait de chaque ligne les coordonnées géographiques de la communes ainsi que le code INSEE
				// qui servira d'index du tableau constitué.
				// On ne sélectionne que les colonnes correspondant à des champs additionnels.
				$enregistrement_geo = $enregistrement_geo_defaut;
				foreach ($titres as $_cle => $_titre) {
					$titre = trim($_titre);
					if (isset($config_champs_geo[$titre]) and !empty($valeurs[$_cle])) {
						$enregistrement_geo[$config_champs_geo[$titre]] = trim($valeurs[$_cle]);
					}
				}
				if (isset($valeurs[$index_code_insee])) {
					$enregistrements_geo[$valeurs[$index_code_insee]] = $enregistrement_geo;
				}
			}
		}
	}

	// On complète maintenant le tableau des enregistrements avec les informations glanées dans les fichiers
	// additionnels.
	foreach ($enregistrements as $_cle => $_enregistrement) {
		// Les coordonnées géographiques
		$code = $_enregistrement['code'];
		if (isset($enregistrements_geo[$code])) {
			$enregistrements[$_cle] = array_merge($enregistrements[$_cle], $enregistrements_geo[$code]);
		} else {
			$enregistrements[$_cle] = array_merge($enregistrements[$_cle], $enregistrement_geo_defaut);
		}
	}

	return $enregistrements;
}
